@@ -19,7 +19,9 @@ package machine
 import (
 	"context"
 	"encoding/base64"
+	"errors"
 	"fmt"
+	"reflect"
 	"strings"
 	"testing"
 
@@ -30,8 +32,7 @@ import (
 	"github.com/golang/mock/gomock"
 	machinev1 "github.com/openshift/machine-api-operator/pkg/apis/machine/v1beta1"
 	"github.com/openshift/machine-api-operator/pkg/controller/machine"
-	controllerError "github.com/openshift/machine-api-operator/pkg/controller/machine"
-	"github.com/pkg/errors"
+	machineapierrors "github.com/openshift/machine-api-operator/pkg/controller/machine"
 	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
@@ -39,8 +40,6 @@ import (
 	"k8s.io/client-go/tools/record"
 	"k8s.io/klog"
 	"k8s.io/utils/pointer"
-	clusterproviderv1 "sigs.k8s.io/cluster-api-provider-azure/pkg/apis/azureprovider/v1alpha1"
-	machineproviderv1 "sigs.k8s.io/cluster-api-provider-azure/pkg/apis/azureprovider/v1beta1"
 	providerspecv1 "sigs.k8s.io/cluster-api-provider-azure/pkg/apis/azureprovider/v1beta1"
 	"sigs.k8s.io/cluster-api-provider-azure/pkg/cloud/azure"
 	"sigs.k8s.io/cluster-api-provider-azure/pkg/cloud/azure/actuators"
@@ -59,14 +58,7 @@ func init() {
 	}
 }
 
-func newClusterProviderSpec() clusterproviderv1.AzureClusterProviderSpec {
-	return clusterproviderv1.AzureClusterProviderSpec{
-		ResourceGroup: "resource-group-test",
-		Location:      "southcentralus",
-	}
-}
-
-func providerSpecFromMachine(in *machineproviderv1.AzureMachineProviderSpec) (*machinev1.ProviderSpec, error) {
+func providerSpecFromMachine(in *providerspecv1.AzureMachineProviderSpec) (*machinev1.ProviderSpec, error) {
 	bytes, err := yaml.Marshal(in)
 	if err != nil {
 		return nil, err
@@ -76,7 +68,7 @@ func providerSpecFromMachine(in *machineproviderv1.AzureMachineProviderSpec) (*m
 	}, nil
 }
 
-func newMachine(t *testing.T, machineConfig machineproviderv1.AzureMachineProviderSpec, labels map[string]string) *machinev1.Machine {
+func newMachine(t *testing.T, machineConfig providerspecv1.AzureMachineProviderSpec, labels map[string]string) *machinev1.Machine {
 	providerSpec, err := providerSpecFromMachine(&machineConfig)
 	if err != nil {
 		t.Fatalf("error encoding provider config: %v", err)
@@ -93,32 +85,22 @@ func newMachine(t *testing.T, machineConfig machineproviderv1.AzureMachineProvid
 }
 
 func newFakeScope(t *testing.T, label string) *actuators.MachineScope {
-	scope := &actuators.Scope{
-		Context: context.Background(),
-		ClusterConfig: &clusterproviderv1.AzureClusterProviderSpec{
-			ResourceGroup:       "dummyResourceGroup",
-			Location:            "dummyLocation",
-			CAKeyPair:           clusterproviderv1.KeyPair{Cert: []byte("cert"), Key: []byte("key")},
-			EtcdCAKeyPair:       clusterproviderv1.KeyPair{Cert: []byte("cert"), Key: []byte("key")},
-			FrontProxyCAKeyPair: clusterproviderv1.KeyPair{Cert: []byte("cert"), Key: []byte("key")},
-			SAKeyPair:           clusterproviderv1.KeyPair{Cert: []byte("cert"), Key: []byte("key")},
-			DiscoveryHashes:     []string{"discoveryhash0"},
-		},
-	}
 	labels := make(map[string]string)
-	labels[machineproviderv1.MachineRoleLabel] = label
+	labels[providerspecv1.MachineRoleLabel] = label
 	labels[machinev1.MachineClusterIDLabel] = "clusterID"
-	machineConfig := machineproviderv1.AzureMachineProviderSpec{}
+	machineConfig := providerspecv1.AzureMachineProviderSpec{}
 	m := newMachine(t, machineConfig, labels)
 	return &actuators.MachineScope{
-		Scope:   scope,
+		Context: context.Background(),
 		Machine: m,
-		MachineConfig: &machineproviderv1.AzureMachineProviderSpec{
+		MachineConfig: &providerspecv1.AzureMachineProviderSpec{
+			ResourceGroup:   "dummyResourceGroup",
+			Location:        "dummyLocation",
 			Subnet:          "dummySubnet",
 			Vnet:            "dummyVnet",
 			ManagedIdentity: "dummyIdentity",
 		},
-		MachineStatus: &machineproviderv1.AzureMachineProviderStatus{},
+		MachineStatus: &providerspecv1.AzureMachineProviderStatus{},
 	}
 }
 
@@ -130,7 +112,7 @@ func newFakeReconciler(t *testing.T) *Reconciler {
 		ProvisioningState: "Succeeded",
 	}
 	return &Reconciler{
-		scope:                 newFakeScope(t, machineproviderv1.ControlPlane),
+		scope:                 newFakeScope(t, providerspecv1.ControlPlane),
 		availabilityZonesSvc:  fakeSuccessSvc,
 		networkInterfacesSvc:  fakeSuccessSvc,
 		virtualMachinesSvc:    fakeVMSuccessSvc,
@@ -404,7 +386,7 @@ func (s *FakeAvailabilityZonesService) Delete(ctx context.Context, spec azure.Sp
 }
 
 func TestAvailabilityZones(t *testing.T) {
-	fakeScope := newFakeScope(t, machineproviderv1.ControlPlane)
+	fakeScope := newFakeScope(t, providerspecv1.ControlPlane)
 	fakeReconciler := newFakeReconcilerWithScope(t, fakeScope)
 
 	fakeReconciler.scope.MachineConfig.Zone = to.StringPtr("2")
@@ -448,7 +430,7 @@ func TestGetZone(t *testing.T) {
 	}
 
 	for _, tc := range testCases {
-		fakeScope := newFakeScope(t, machineproviderv1.ControlPlane)
+		fakeScope := newFakeScope(t, providerspecv1.ControlPlane)
 		fakeReconciler := newFakeReconcilerWithScope(t, fakeScope)
 		fakeReconciler.scope.MachineConfig.Zone = tc.inputZone
 
@@ -469,18 +451,18 @@ func TestGetZone(t *testing.T) {
 }
 
 func TestCustomUserData(t *testing.T) {
-	fakeScope := newFakeScope(t, machineproviderv1.Node)
+	fakeScope := newFakeScope(t, providerspecv1.Node)
 	userDataSecret := &corev1.Secret{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:      "testCustomUserData",
-			Namespace: "dummyNamespace",
+			Namespace: fakeScope.Namespace(),
 		},
 		Data: map[string][]byte{
 			"userData": []byte("test-userdata"),
 		},
 	}
 	fakeScope.CoreClient = controllerfake.NewFakeClient(userDataSecret)
-	fakeScope.MachineConfig.UserDataSecret = &corev1.SecretReference{Name: "testCustomUserData"}
+	fakeScope.MachineConfig.UserDataSecret = &corev1.SecretReference{Name: userDataSecret.Name, Namespace: fakeScope.Namespace()}
 	fakeReconciler := newFakeReconcilerWithScope(t, fakeScope)
 	fakeReconciler.virtualMachinesSvc = &FakeVMCheckZonesService{}
 	if err := fakeReconciler.Create(context.Background()); err != nil {
@@ -498,7 +480,7 @@ func TestCustomUserData(t *testing.T) {
 }
 
 func TestCustomDataFailures(t *testing.T) {
-	fakeScope := newFakeScope(t, machineproviderv1.Node)
+	fakeScope := newFakeScope(t, providerspecv1.Node)
 	userDataSecret := &corev1.Secret{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:      "testCustomUserData",
@@ -568,7 +550,7 @@ func TestMachineEvents(t *testing.T) {
 			operation: func(actuator *Actuator, machine *machinev1.Machine) {
 				actuator.Create(context.TODO(), machine)
 			},
-			event: "Warning FailedCreate InvalidConfiguration: failed to create machine \"azure-actuator-testing-machine\" scope: failed to update cluster: Azure client id /azure-credentials-secret did not contain key azure_client_id",
+			event: "Warning FailedCreate InvalidConfiguration: failed to create machine \"azure-actuator-testing-machine\" scope: failed to update cluster: Azure client id default/azure-credentials-secret did not contain key azure_client_id",
 		},
 		{
 			name:       "Create machine event failed (reconciler)",
@@ -577,7 +559,7 @@ func TestMachineEvents(t *testing.T) {
 			operation: func(actuator *Actuator, machine *machinev1.Machine) {
 				actuator.Create(context.TODO(), machine)
 			},
-			event: "Warning FailedCreate CreateError: failed to reconcile machine \"azure-actuator-testing-machine\"s: failed to create nic azure-actuator-testing-machine-nic for machine azure-actuator-testing-machine: MachineConfig vnet is missing on machine azure-actuator-testing-machine",
+			event: "Warning FailedCreate InvalidConfiguration: failed to reconcile machine \"azure-actuator-testing-machine\": failed to create nic azure-actuator-testing-machine-nic for machine azure-actuator-testing-machine: MachineConfig vnet is missing on machine azure-actuator-testing-machine",
 		},
 		{
 			name:       "Create machine event succeed",
@@ -595,7 +577,7 @@ func TestMachineEvents(t *testing.T) {
 			operation: func(actuator *Actuator, machine *machinev1.Machine) {
 				actuator.Update(context.TODO(), machine)
 			},
-			event: "Warning FailedUpdate UpdateError: failed to create machine \"azure-actuator-testing-machine\" scope: failed to update cluster: Azure client id /azure-credentials-secret did not contain key azure_client_id",
+			event: "Warning FailedUpdate UpdateError: failed to create machine \"azure-actuator-testing-machine\" scope: failed to update cluster: Azure client id default/azure-credentials-secret did not contain key azure_client_id",
 		},
 		{
 			name:       "Update machine event succeed",
@@ -613,7 +595,7 @@ func TestMachineEvents(t *testing.T) {
 			operation: func(actuator *Actuator, machine *machinev1.Machine) {
 				actuator.Delete(context.TODO(), machine)
 			},
-			event: "Warning FailedDelete DeleteError: failed to create machine \"azure-actuator-testing-machine\" scope: failed to update cluster: Azure client id /azure-credentials-secret did not contain key azure_client_id",
+			event: "Warning FailedDelete DeleteError: failed to create machine \"azure-actuator-testing-machine\" scope: failed to update cluster: Azure client id default/azure-credentials-secret did not contain key azure_client_id",
 		},
 		{
 			name:       "Delete machine event failed (reconciler)",
@@ -639,7 +621,8 @@ func TestMachineEvents(t *testing.T) {
 		t.Run(tc.name, func(t *testing.T) {
 			cs := controllerfake.NewFakeClientWithScheme(scheme.Scheme, tc.credSecret)
 
-			if err := cs.Create(context.TODO(), machine); err != nil {
+			m := tc.machine.DeepCopy()
+			if err := cs.Create(context.TODO(), m); err != nil {
 				t.Fatal(err)
 			}
 
@@ -691,7 +674,7 @@ func TestMachineEvents(t *testing.T) {
 			networkSvc.EXPECT().Delete(gomock.Any(), gomock.Any()).Return(nil).AnyTimes()
 			pipSvc.EXPECT().Delete(gomock.Any(), gomock.Any()).Return(nil).AnyTimes()
 
-			tc.operation(machineActuator, tc.machine)
+			tc.operation(machineActuator, m)
 
 			select {
 			case event := <-eventsChannel:
@@ -726,7 +709,7 @@ func TestStatusCodeBasedCreationErrors(t *testing.T) {
 		},
 		{
 			name:       "CreateMachine",
-			event:      "Warning FailedCreate CreateError: failed to reconcile machine \"azure-actuator-testing-machine\"s: failed to create vm azure-actuator-testing-machine : failed to create or get machine: failed to create or get machine: compute.VirtualMachinesClient#CreateOrUpdate: MOCK: StatusCode=300",
+			event:      "Warning FailedCreate CreateError: failed to reconcile machine \"azure-actuator-testing-machine\"s: failed to create vm azure-actuator-testing-machine: failed to create or get machine: failed to create or get machine: compute.VirtualMachinesClient#CreateOrUpdate: MOCK: StatusCode=300",
 			statusCode: 300,
 			requeable:  true,
 		},
@@ -768,11 +751,11 @@ func TestStatusCodeBasedCreationErrors(t *testing.T) {
 			networkSvc.EXPECT().CreateOrUpdate(gomock.Any(), gomock.Any()).Return(nil).Times(1)
 			azureErr := autorest.NewError("compute.VirtualMachinesClient", "CreateOrUpdate", "MOCK")
 			azureErr.StatusCode = tc.statusCode
-			wrapErr := errors.Wrapf(azureErr, "failed to create or get machine")
+			wrapErr := fmt.Errorf("failed to create or get machine: %w", azureErr)
 			vmSvc.EXPECT().CreateOrUpdate(gomock.Any(), gomock.Any()).Return(wrapErr).Times(1)
 			vmSvc.EXPECT().Get(gomock.Any(), gomock.Any()).Return(nil, autorest.NewError("compute.VirtualMachinesClient", "Get", "MOCK")).Times(1)
 
-			_, ok := machineActuator.Create(context.TODO(), machine).(*controllerError.RequeueAfterError)
+			_, ok := machineActuator.Create(context.TODO(), machine).(*machineapierrors.RequeueAfterError)
 			if ok && !tc.requeable {
 				t.Error("Error is not requeable but was requeued")
 			}
@@ -791,3 +774,97 @@ func TestStatusCodeBasedCreationErrors(t *testing.T) {
 		})
 	}
 }
+
+func TestInvalidConfigurationCreationErrors(t *testing.T) {
+	cases := []struct {
+		name          string
+		mutateMachine func(*machinev1.Machine)
+		mutatePC      func(*providerspecv1.AzureMachineProviderSpec)
+		expectedErr   error
+	}{
+		{
+			name: "Machine Name too long with PublicIP",
+			mutateMachine: func(m *machinev1.Machine) {
+				m.Name = "MachineNameOverSixtyChars" + "abcdefghijklmnopqrstuvwxyz" + "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+			},
+			mutatePC: func(s *providerspecv1.AzureMachineProviderSpec) {
+				s.PublicIP = true
+			},
+			expectedErr: machineapierrors.InvalidMachineConfiguration("failed to reconcile machine \"MachineNameOverSixtyCharsabcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ\": failed to create nic MachineNameOverSixtyCharsabcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-nic for machine MachineNameOverSixtyCharsabcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ: unable to create Public IP: machine public IP name is longer than 63 characters"),
+		},
+		{
+			name: "Machine Config missing vnet",
+			mutatePC: func(s *providerspecv1.AzureMachineProviderSpec) {
+				s.Vnet = ""
+			},
+			expectedErr: machineapierrors.InvalidMachineConfiguration("failed to reconcile machine \"azure-actuator-testing-machine\": failed to create nic azure-actuator-testing-machine-nic for machine azure-actuator-testing-machine: MachineConfig vnet is missing on machine azure-actuator-testing-machine"),
+		},
+		{
+			name: "Machine Config missing subnet",
+			mutatePC: func(s *providerspecv1.AzureMachineProviderSpec) {
+				s.Subnet = ""
+			},
+			expectedErr: machineapierrors.InvalidMachineConfiguration("failed to reconcile machine \"azure-actuator-testing-machine\": failed to create nic azure-actuator-testing-machine-nic for machine azure-actuator-testing-machine: MachineConfig subnet is missing on machine azure-actuator-testing-machine, skipping machine creation"),
+		},
+	}
+
+	for _, tc := range cases {
+		t.Run(tc.name, func(t *testing.T) {
+			cs := controllerfake.NewFakeClient(stubAzureCredentialsSecret(), &corev1.Secret{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "azure-actuator-user-data-secret",
+					Namespace: "default",
+				},
+				Data: map[string][]byte{
+					"userData": []byte("S3CR3T"),
+				},
+			})
+
+			mockCtrl := gomock.NewController(t)
+			networkSvc := mock_azure.NewMockService(mockCtrl)
+			vmSvc := mock_azure.NewMockService(mockCtrl)
+
+			eventsChannel := make(chan string, 1)
+
+			machineActuator := NewActuator(ActuatorParams{
+				CoreClient: cs,
+				ReconcilerBuilder: func(scope *actuators.MachineScope) *Reconciler {
+					return &Reconciler{
+						scope:                scope,
+						networkInterfacesSvc: networkSvc,
+						virtualMachinesSvc:   vmSvc,
+					}
+				},
+				// use fake recorder and store an event into one item long buffer for subsequent check
+				EventRecorder: &record.FakeRecorder{
+					Events: eventsChannel,
+				},
+			})
+
+			machine, err := stubMachine()
+			if err != nil {
+				t.Fatal(err)
+			}
+
+			if tc.mutateMachine != nil {
+				tc.mutateMachine(machine)
+			}
+
+			if tc.mutatePC != nil {
+				machinePC := stubProviderConfig()
+				tc.mutatePC(machinePC)
+
+				providerSpec, err := providerspecv1.EncodeMachineSpec(machinePC)
+				if err != nil {
+					t.Fatalf("unexpected error encoding providerspec: %v", err)
+				}
+				machine.Spec.ProviderSpec.Value = providerSpec
+			}
+
+			err = machineActuator.Create(context.TODO(), machine)
+			if !reflect.DeepEqual(err, tc.expectedErr) {
+				t.Errorf("Expected: %#v, Got: %#v", tc.expectedErr, err)
+			}
+		})
+	}
+}
