@@ -18,28 +18,34 @@ package networkinterfaces
 
 import (
 	"context"
+	"errors"
+	"fmt"
+	"net"
 
 	"github.com/Azure/azure-sdk-for-go/services/network/mgmt/2018-12-01/network"
 	"github.com/Azure/go-autorest/autorest/to"
-	"github.com/pkg/errors"
 	"k8s.io/klog"
 	"sigs.k8s.io/cluster-api-provider-azure/pkg/cloud/azure"
+	"sigs.k8s.io/cluster-api-provider-azure/pkg/cloud/azure/services/applicationsecuritygroups"
 	"sigs.k8s.io/cluster-api-provider-azure/pkg/cloud/azure/services/internalloadbalancers"
 	"sigs.k8s.io/cluster-api-provider-azure/pkg/cloud/azure/services/publicips"
 	"sigs.k8s.io/cluster-api-provider-azure/pkg/cloud/azure/services/publicloadbalancers"
+	"sigs.k8s.io/cluster-api-provider-azure/pkg/cloud/azure/services/securitygroups"
 	"sigs.k8s.io/cluster-api-provider-azure/pkg/cloud/azure/services/subnets"
 )
 
 // Spec specification for routetable
 type Spec struct {
-	Name                     string
-	SubnetName               string
-	VnetName                 string
-	StaticIPAddress          string
-	PublicLoadBalancerName   string
-	InternalLoadBalancerName string
-	NatRule                  *int
-	PublicIP                 string
+	Name                          string
+	SubnetName                    string
+	VnetName                      string
+	StaticIPAddress               string
+	PublicLoadBalancerName        string
+	InternalLoadBalancerName      string
+	NatRule                       *int
+	PublicIP                      string
+	SecurityGroupName             string
+	ApplicationSecurityGroupNames []string
 }
 
 // Get provides information about a network interface.
@@ -48,9 +54,9 @@ func (s *Service) Get(ctx context.Context, spec azure.Spec) (interface{}, error)
 	if !ok {
 		return network.Interface{}, errors.New("invalid network interface specification")
 	}
-	nic, err := s.Client.Get(ctx, s.Scope.ClusterConfig.ResourceGroup, nicSpec.Name, "")
+	nic, err := s.Client.Get(ctx, s.Scope.MachineConfig.ResourceGroup, nicSpec.Name, "")
 	if err != nil && azure.ResourceNotFound(err) {
-		return nil, errors.Wrapf(err, "network interface %s not found", nicSpec.Name)
+		return nil, fmt.Errorf("network interface %s not found: %w", nicSpec.Name, err)
 	} else if err != nil {
 		return nic, err
 	}
@@ -64,6 +70,7 @@ func (s *Service) CreateOrUpdate(ctx context.Context, spec azure.Spec) error {
 		return errors.New("invalid network interface specification")
 	}
 
+	nicProp := network.InterfacePropertiesFormat{}
 	nicConfig := &network.InterfaceIPConfigurationPropertiesFormat{}
 
 	subnetInterface, err := subnets.NewService(s.Scope).Get(ctx, &subnets.Spec{Name: nicSpec.SubnetName, VnetName: nicSpec.VnetName})
@@ -137,33 +144,84 @@ func (s *Service) CreateOrUpdate(ctx context.Context, spec azure.Spec) error {
 	}
 	nicConfig.LoadBalancerBackendAddressPools = &backendAddressPools
 
+	if nicSpec.SecurityGroupName != "" {
+		securityGroupInterface, sgerr := securitygroups.NewService(s.Scope).Get(ctx, &securitygroups.Spec{Name: nicSpec.SecurityGroupName})
+		if sgerr != nil {
+			return sgerr
+		}
+
+		sg, ok := securityGroupInterface.(network.SecurityGroup)
+		if !ok {
+			return errors.New("security group get returned invalid network interface")
+		}
+		nicProp.NetworkSecurityGroup = &network.SecurityGroup{ID: sg.ID}
+	}
+
+	if len(nicSpec.ApplicationSecurityGroupNames) > 0 {
+		asgSvc := applicationsecuritygroups.NewService(s.Scope)
+
+		var groups []network.ApplicationSecurityGroup
+		for _, asgName := range nicSpec.ApplicationSecurityGroupNames {
+			asgInterface, asgerr := asgSvc.Get(ctx, &applicationsecuritygroups.Spec{Name: asgName})
+			if err != nil {
+				return asgerr
+			}
+			asg, ok := asgInterface.(network.ApplicationSecurityGroup)
+			if !ok {
+				return errors.New("application security group get returned invalid network interface")
+			}
+			groups = append(groups, network.ApplicationSecurityGroup{
+				ID: asg.ID,
+			})
+		}
+		nicConfig.ApplicationSecurityGroups = &groups
+	}
+
+	nicProp.IPConfigurations = &[]network.InterfaceIPConfiguration{
+		{
+			Name:                                     to.StringPtr("pipConfig"),
+			InterfaceIPConfigurationPropertiesFormat: nicConfig,
+		},
+	}
+
+	if subnetHasIPv6(subnet) {
+		klog.V(2).Infof("Found IPv6 address space. Adding IPv6 configuration to nic: %s", nicSpec.Name)
+		nicConfigIPv6 := &network.InterfaceIPConfigurationPropertiesFormat{}
+
+		nicConfigIPv6.Subnet = &network.Subnet{ID: subnet.ID}
+		nicConfigIPv6.PrivateIPAllocationMethod = network.Dynamic
+		nicConfigIPv6.PrivateIPAddressVersion = network.IPv6
+
+		ipConfigs := append(*nicProp.IPConfigurations,
+			network.InterfaceIPConfiguration{
+				Name:                                     to.StringPtr("pipConfig-v6"),
+				InterfaceIPConfigurationPropertiesFormat: nicConfigIPv6,
+			},
+		)
+
+		nicProp.IPConfigurations = &ipConfigs
+	}
+
 	f, err := s.Client.CreateOrUpdate(ctx,
-		s.Scope.ClusterConfig.ResourceGroup,
+		s.Scope.MachineConfig.ResourceGroup,
 		nicSpec.Name,
 		network.Interface{
-			Location: to.StringPtr(s.Scope.ClusterConfig.Location),
-			InterfacePropertiesFormat: &network.InterfacePropertiesFormat{
-				IPConfigurations: &[]network.InterfaceIPConfiguration{
-					{
-						Name:                                     to.StringPtr("pipConfig"),
-						InterfaceIPConfigurationPropertiesFormat: nicConfig,
-					},
-				},
-			},
+			Location:                  to.StringPtr(s.Scope.MachineConfig.Location),
+			InterfacePropertiesFormat: &nicProp,
 		})
 
 	if err != nil {
-		return errors.Wrapf(err, "failed to create network interface %s in resource group %s", nicSpec.Name, s.Scope.ClusterConfig.ResourceGroup)
+		return fmt.Errorf("failed to create network interface %s in resource group %s: %w", nicSpec.Name, s.Scope.MachineConfig.ResourceGroup, err)
 	}
 
 	err = f.WaitForCompletionRef(ctx, s.Client.Client)
 	if err != nil {
-		return errors.Wrap(err, "cannot create, future response")
+		return fmt.Errorf("cannot create, future response: %w", err)
 	}
 
 	_, err = f.Result(s.Client)
 	if err != nil {
-		return errors.Wrap(err, "result error")
+		return fmt.Errorf("result error: %w", err)
 	}
 	klog.V(2).Infof("successfully created network interface %s", nicSpec.Name)
 	return err
@@ -176,24 +234,50 @@ func (s *Service) Delete(ctx context.Context, spec azure.Spec) error {
 		return errors.New("invalid network interface Specification")
 	}
 	klog.V(2).Infof("deleting nic %s", nicSpec.Name)
-	f, err := s.Client.Delete(ctx, s.Scope.ClusterConfig.ResourceGroup, nicSpec.Name)
+	f, err := s.Client.Delete(ctx, s.Scope.MachineConfig.ResourceGroup, nicSpec.Name)
 	if err != nil && azure.ResourceNotFound(err) {
 		// already deleted
 		return nil
 	}
 	if err != nil {
-		return errors.Wrapf(err, "failed to delete network interface %s in resource group %s", nicSpec.Name, s.Scope.ClusterConfig.ResourceGroup)
+		return fmt.Errorf("failed to delete network interface %s in resource group %s: %w", nicSpec.Name, s.Scope.MachineConfig.ResourceGroup, err)
 	}
 
 	err = f.WaitForCompletionRef(ctx, s.Client.Client)
 	if err != nil {
-		return errors.Wrap(err, "cannot create, future response")
+		return fmt.Errorf("cannot create, future response: %w", err)
 	}
 
 	_, err = f.Result(s.Client)
 	if err != nil {
-		return errors.Wrap(err, "result error")
+		return fmt.Errorf("result error: %w", err)
 	}
 	klog.V(2).Infof("successfully deleted nic %s", nicSpec.Name)
 	return err
 }
+
+func subnetHasIPv6(subnet network.Subnet) bool {
+	var prefixes []string
+
+	if subnet.AddressPrefix != nil {
+		prefixes = append(prefixes, *subnet.AddressPrefix)
+	}
+
+	if subnet.AddressPrefixes != nil {
+		prefixes = append(prefixes, *subnet.AddressPrefixes...)
+	}
+
+	for _, prefix := range prefixes {
+		ip, _, err := net.ParseCIDR(prefix)
+		if err != nil {
+			klog.Errorf("Error parsing subnet address prefix: %v", err)
+			continue
+		}
+
+		if ip != nil && ip.To4() == nil {
+			return true // Found an IPv6 subnet.
+		}
+	}
+
+	return false
+}
